@@ -480,15 +480,15 @@ from data.models.models import ChannelStats, DataTransformer, DataTransformerCon
 #                                                      extra=1,exclude =['updated_at','created_at'],formset=DataTransformerForm)
 
 class DataConstantsTransformerForm(forms.ModelForm):
-    id = forms.UUIDField(required=False,
+    id = forms.UUIDField(required=True,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}),initial=uuid.uuid4)
 
     index = forms.IntegerField(required=False,
                                widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                                initial=0)
-    value = forms.CharField(required=False,
+    value = forms.CharField(required=True,
                             widget=forms.TextInput(attrs={'class': 'form-control'}))
-    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
+    data_transformer = forms.ModelChoiceField(required=True, queryset=DataTransformer.objects.all(),
                                               widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     class Meta:
@@ -557,7 +557,7 @@ class BaseInlineFormSetWithAddMoreButton(BaseInlineFormSet):
 
 
 DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformerConstants, formset=BaseInlineFormSetWithAddMoreButton, extra=0,
-                                                              form=DataConstantsTransformerForm, can_delete_extra=True)
+                                                              form=DataConstantsTransformerForm, can_delete_extra=True, fk_name='data_transformer')
 # HardwareTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareDataTransformer, extra=1,
 #                                                          form=HardwareTransformerForm, can_delete_extra=True)
 # HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=0, formset=BaseInlineFormSetWithAddMoreButton,
@@ -631,8 +631,7 @@ class DataTransform(forms.ModelForm):
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': str(self.instance.id),
-                              'id': str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.id)}],
                     prefix=nested_data_transformer_form_prefix,
                 )
                 self.nested_data_transformer_form.extra = self.instance.get_children().count()
@@ -644,8 +643,7 @@ class DataTransform(forms.ModelForm):
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.id),
-                              'id': str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.id)}],
                 )
                 self.nested_data_transformer_form.extra = 0
         except Exception as e:
@@ -653,24 +651,23 @@ class DataTransform(forms.ModelForm):
             if nested_data_transformer_form_data is not None and len(nested_data_transformer_form_data.keys()) > 0:
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
-                    data=self.data if self.is_bound else None,
+                    data=self.data,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.id),
-                              'id': str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.id)}],
                 )
                 self.nested_data_transformer_form.extra = 0
 
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
             instance=self.instance,
             queryset=self.instance.data_transformer_constants,
-            data=self.data if self.is_bound else None,
+            data=self.data,
             files=self.files if self.is_bound else None,
             prefix=nested_constants_form_prefix,
-            initial=[{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())}]
+            initial=[{'data_transformer': str(self.instance.id)},{'data_transformer': str(self.instance.id)},{'data_transformer': str(self.instance.id)}]
 
         )
-        self.nested_constants_form.extra = self.instance.data_transformer_constants.count()
+        self.nested_constants_form.extra = 0
 
         # hardware_outputs = self.instance.hardwaredatatransformer_set.first()
         # self.nested_hardware_outputs_form = HardwareTransformerForm(
@@ -702,10 +699,10 @@ class DataTransform(forms.ModelForm):
 
         if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
             forms = forms + '\n  -' + self.nested_constants_form.as_p()
-        if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
-            forms = forms + '\n  -' + self.nested_hardware_outputs_form.as_p()
-        if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
-            forms = forms + '\n  -' + self.nested_hardware_stats_form.as_p()
+        # if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+        #     forms = forms + '\n  -' + self.nested_hardware_outputs_form.as_p()
+        # if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
+        #     forms = forms + '\n  -' + self.nested_hardware_stats_form.as_p()
         if hasattr(self, 'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
             print('netdatatransform with number elements: ' + str(len(self.nested_data_transformer_form)))
             forms = forms + '\n       -' + self.nested_data_transformer_form.as_p()
@@ -724,21 +721,21 @@ class DataTransform(forms.ModelForm):
                 self.nested_constants_form.save(commit)
             else:
                 print(self.nested_constants_form.errors)
-        if hasattr(self,
-                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
-
-            if self.nested_hardware_outputs_form.is_valid():
-                self.nested_hardware_outputs_form.save(commit)
-            else:
-                print(self.nested_hardware_outputs_form.errors)
-        if hasattr(self,
-                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
-            self.nested_hardware_stats_form.instance.refresh_from_db()
-
-            if self.nested_hardware_stats_form.is_valid():
-                self.nested_hardware_stats_form.save(commit)
-            else:
-                print(self.nested_hardware_stats_form.errors)
+        # if hasattr(self,
+        #            'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+        #
+        #     if self.nested_hardware_outputs_form.is_valid():
+        #         self.nested_hardware_outputs_form.save(commit)
+        #     else:
+        #         print(self.nested_hardware_outputs_form.errors)
+        # if hasattr(self,
+        #            'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
+        #     self.nested_hardware_stats_form.instance.refresh_from_db()
+        #
+        #     if self.nested_hardware_stats_form.is_valid():
+        #         self.nested_hardware_stats_form.save(commit)
+        #     else:
+        #         print(self.nested_hardware_stats_form.errors)
         if hasattr(self,
                    'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
             self.nested_data_transformer_form.instance.refresh_from_db()
@@ -752,7 +749,7 @@ class DataTransform(forms.ModelForm):
 
     class Meta:
         model = DataTransformer
-        fields = ['id', 'accessory', 'parent', 'type']
+        fields = ['id', 'accessory', 'parent','type', 'channels','channel_stats']
 
 
 # DataTransformerFormset = form_factory(DataTransformer,formset=DataTransformerForm)
@@ -767,7 +764,7 @@ dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransf
 
 
 class DataTransformRoot(forms.ModelForm):
-    id = forms.UUIDField(required=False,
+    id = forms.UUIDField(required=True,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}),initial=uuid.uuid4)
     accessory = forms.ModelChoiceField(required=True, queryset=Accessory.objects.all(),
                                        widget=forms.HiddenInput(attrs={'class': 'form-control'}))
@@ -792,15 +789,14 @@ class DataTransformRoot(forms.ModelForm):
         nested_hardware_stats_form_data = None
 
         if 'data' in kwargs.keys():
-            if 'data' in kwargs.keys():
-                query_dict = kwargs['data'].dict()
-                form_data = {}
-                if 'id' in query_dict:
-                    form_data['id'] = query_dict['id']
-                if 'accessory' in query_dict:
-                    form_data['accessory'] = query_dict['accessory']
-                if 'type' in query_dict:
-                    form_data['type'] = query_dict['type']
+            query_dict = kwargs['data'].dict()
+            form_data = {}
+            if 'id' in query_dict:
+                form_data['id'] = query_dict['id']
+            if 'accessory' in query_dict:
+                form_data['accessory'] = query_dict['accessory']
+            if 'type' in query_dict:
+                form_data['type'] = query_dict['type']
 
             nested_data_transformer_form_data = get_values_for_keys_containing(nested_data_transformer_form_prefix,
                                                                                query_dict)
@@ -823,8 +819,7 @@ class DataTransformRoot(forms.ModelForm):
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
-                              'id': str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id)}],
                     prefix=nested_data_transformer_form_prefix,
                 )
                 self.nested_data_transformer_form.extra = self.instance.get_children().count()
@@ -833,11 +828,10 @@ class DataTransformRoot(forms.ModelForm):
 
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
-                    data=self.data if self.is_bound else None,
+                    data=self.data,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.pk),'accessory': str(self.instance.accessory.id),
-                              'id': str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.pk),'accessory': str(self.instance.accessory.id)}],
                 )
                 self.nested_data_transformer_form.extra = 0
         except Exception as e:
@@ -845,11 +839,10 @@ class DataTransformRoot(forms.ModelForm):
             if nested_data_transformer_form_data is not None and len(nested_data_transformer_form_data.keys())>0:
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
-                    data=self.data if self.is_bound else None,
+                    data=self.data,
                     files=self.files if self.is_bound else None,
                     prefix=nested_data_transformer_form_prefix,
-                    initial=[{'parent': str(self.instance.pk),'accessory': str(self.instance.accessory.id),
-                              'id': str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.pk),'accessory': str(self.instance.accessory.id)}],
                 )
                 self.nested_data_transformer_form.extra = 0
 
@@ -857,10 +850,10 @@ class DataTransformRoot(forms.ModelForm):
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
             instance=self.instance,
             queryset=self.instance.data_transformer_constants,
-            data=self.data if self.is_bound else None,
+            data=self.data,
             files=self.files if self.is_bound else None,
             prefix=nested_constants_form_prefix,
-            initial=[{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())},{'data_transformer': str(self.instance.id), 'id': str(uuid.uuid4())}]
+            initial=[{'data_transformer': str(self.instance.id)},{'data_transformer': str(self.instance.id)},{'data_transformer': str(self.instance.id)},{'data_transformer': str(self.instance.id)}]
 
         )
         self.nested_constants_form.extra = self.instance.data_transformer_constants.count()
@@ -894,10 +887,10 @@ class DataTransformRoot(forms.ModelForm):
         forms = forms + '</div><div class="col-6">'
         if hasattr(self, 'nested_constants_form') and self.nested_constants_form is not None:
             forms = forms + '\n  -' + self.nested_constants_form.as_p()
-        if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
-            forms = forms + '\n  -' + self.nested_hardware_outputs_form.as_p()
-        if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
-            forms = forms + '\n  -' + self.nested_hardware_stats_form.as_p()
+        # if hasattr(self, 'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+        #     forms = forms + '\n  -' + self.nested_hardware_outputs_form.as_p()
+        # if hasattr(self, 'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
+        #     forms = forms + '\n  -' + self.nested_hardware_stats_form.as_p()
         if hasattr(self, 'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
             print('netdatatransform with number elements: ' + str(len(self.nested_data_transformer_form)))
             forms = forms + '\n       -' + self.nested_data_transformer_form.as_p()
