@@ -223,6 +223,9 @@ class DataTransformerConstantsSerializer(serializers.ModelSerializer):
 
 class DataTransformerTreeSerializer(serializers.ModelSerializer):
     data_transformer_constants = DataTransformerConstantsSerializer(many=True)
+    channels = serializers.PrimaryKeyRelatedField(read_only=True,many=True)
+    channel_stats = serializers.PrimaryKeyRelatedField(read_only=True,many=True)
+
     children = serializers.SerializerMethodField(source='get_children')
     class Meta:
         model=DataTransformer
